
import { FastifyReply } from 'fastify';
// import { redirect_302_found } from '../http';
import { BaseClient, Issuer, TypeOfGenericClient } from 'openid-client';
import { PKCECookieConfig } from './pkce-cookie';

const scopes = 'openid profile email';

export interface OIDCConfig {
	server_url: string;
	signing_algorithm: 'ES512';
	client_id: string;
	client_secret: string;
	pkce: PKCECookieConfig;
}

export function validate_oidc_conf(conf: unknown) : asserts conf is OIDCConfig {
	// todo: validate config
}

export class OIDCProvider {
	#conf: OIDCConfig;
	#issuer: Issuer;
	#Client: TypeOfGenericClient<BaseClient>;
	#client: BaseClient;
	#init_promise: Promise<void>;

	constructor(conf: OIDCConfig) {
		this.#conf = conf;
		this.#init_promise = this.#init();
	}

	public get ready() {
		return this.#init_promise;
	}
	
	async #init() {
		this.#issuer = await Issuer.discover(this.#conf.server_url);
		this.#Client = this.#issuer.Client;
		this.#client = new this.#Client({
			client_id: this.#conf.client_id,
			client_secret: this.#conf.client_secret,
			id_token_signed_response_alg: this.#conf.signing_algorithm,
			authorization_signed_response_alg: this.#conf.signing_algorithm,
		});
	}

	async redirect_to_auth_endpoint(res: FastifyReply, code_challenge: string, redirect_uri: string) {
		await this.#init_promise;

		const uri = this.#client.authorizationUrl({
			response_type: 'code',
			scope: scopes,
			redirect_uri,
			code_challenge,
			code_challenge_method: 'S256'
		});
	
		// todo: redirect
		// return redirect_302_found(res, uri, 'Logging in with OpenID Connect');
	}
}
