
import { createHash } from 'crypto';
import { rand } from '../utilities/rand';
import { set_cookie } from '../utilities/http-cookies';
import { FastifyReply } from 'fastify';

export interface PKCECookieConfig {
	name: string;
	secure: boolean;
	ttl: number;
}

export function validate_pkce_cookie_conf(conf: unknown) : asserts conf is PKCECookieConfig {
	// todo: validate config
}

export class PKCECookieProvider {
	#conf: PKCECookieConfig;
	
	constructor(conf: PKCECookieConfig) {
		this.#conf = conf;
	}

	async setup_pkce_challenge(res: FastifyReply) {
		const pkce_code_verifier = await generate_code_verifier();
		const pkce_code_challenge = sha256_base64_url(pkce_code_verifier);
		const pkce_expire = new Date(Date.now() + (this.#conf.ttl * 1000));
		
		// "Lax" rather than "Strict", so the PKCE verifier will be included on the
		// redirect to the login callback endpoint, which comes from the OpenID server,
		// which is likely on a different site
		set_cookie(res, this.#conf.name, pkce_code_verifier, pkce_expire, this.#conf.secure, 'Lax');
		
		return pkce_code_challenge;
	}
}


async function generate_code_verifier() {
	const base64 = await rand(50, 'base64');
	return replace_base64_url_chars(base64);
}

function sha256_base64_url(input: string) {
	const hash = createHash('sha256');
	hash.update(input);
	const base64 = hash.digest('base64');
	return replace_base64_url_chars(base64);
}

function replace_base64_url_chars(input: string) {
	return input.replace(/\+/g, '-').replace(/\//g, '_').replace(/=+$/, '')
}
